package gregapi.data;

import gregapi.api.Abstract_Mod;
import gregapi.block.IBlockPlacable;
import gregapi.block.prefixblock.PrefixBlock;
import gregapi.code.*;
import gregapi.config.Config;
import gregapi.item.MultiItemTool;
import gregapi.network.INetworkHandler;
import gregapi.old.interfaces.internal.IGT_RecipeAdder;
import gregapi.old.interfaces.internal.IGT_RecipeAdder.NonWorkingRecipeAdder;
import gregapi.oredict.OreDictItemData;
import gregapi.oredict.OreDictMaterial;
import gregapi.oredict.OreDictMaterialStack;
import gregapi.recipes.Recipe;
import gregapi.render.IIconContainer;
import gregapi.util.UT;

import java.io.File;
import java.util.*;

import net.minecraft.block.Block;
import net.minecraft.init.Blocks;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.world.World;
import net.minecraftforge.common.util.ForgeDirection;
import net.minecraftforge.fluids.FluidStack;
import net.minecraftforge.fluids.FluidTank;
import net.minecraftforge.fluids.FluidTankInfo;
import net.minecraftforge.fluids.IFluidTank;
import net.minecraftforge.oredict.OreDictionary;

/**
 * @author Gregorius Techneticies
 * 
 * Class containing useful Constants and Lists.
 * 
 * Mainly made for use with static imports.
 * I am doing this to have a better Table alike view on my Code, so I can change things faster using the Block Selection Mode of eclipse.
 * 
 * For static imports in Eclipse go to "Window > Preferences > Java > Editor > Content Assist > Favorites" to set static importable Constant Classes such as this one as AutoCompleteable.
 */
public class CS {
	// unused: A, D, E, G, H, I, J, K, M, N, O, P, Q, R, S, T
	
	/** Because "true" and "false" are too long. Some Programmers might wanna kill me for that, but this looks much better than true and false, and also it is better to have something that is not 4 and 5 Characters long, because of symmetry */
	public static final boolean T = true, F = false;
	
	/** Character Set with all Numbers */
	public static final HashSet<Character> CN = new HashSet<Character>(Arrays.asList('0', '1', '2', '3', '4', '5', '6', '7', '8', '9'));
	/** Character Set with all lowercased Characters */
	public static final HashSet<Character> CL = new HashSet<Character>(Arrays.asList('a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z'));
	/** Character Set with all uppercased Characters */
	public static final HashSet<Character> CU = new HashSet<Character>(Arrays.asList('A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z'));
	
	/** The first 32 Bits */
	public static final int[] B = new int[] {1<<0,1<<1,1<<2,1<<3,1<<4,1<<5,1<<6,1<<7,1<<8,1<<9,1<<10,1<<11,1<<12,1<<13,1<<14,1<<15,1<<16,1<<17,1<<18,1<<19,1<<20,1<<21,1<<22,1<<23,1<<24,1<<25,1<<26,1<<27,1<<28,1<<29,1<<30,1<<31};
	
	/** 
	 * Renamed from "MATERIAL_UNIT" to just "U"
	 * 
	 * This is worth exactly one normal Item.
	 * This Constant can be divided by many commonly used Numbers such as 
	 * 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 20, 22, 24, 25, ... 64, 81 or 96
	 * without loosing precision and is for that reason used as Unit of Amount.
	 * But it is also small enough to be multiplied with larger Numbers.
	 * 
	 * This is used to determine the amount of Material contained inside a prefixed Ore.
	 * For example Nugget = U / 9 as it contains out of 1/9th of an Ingot.
	 */
	public static final long U = 420076800;
	/** The Double Version of the Material Unit "U" */
	public static final double UD = U;
	
	/**
	 * Renamed from "FLUID_MATERIAL_UNIT" to just "L"
	 * 
	 * Fluid per Material Unit (Prime Factors: 3 * 3 * 2 * 2 * 2 * 2)
	 */
	public static final long L = 144;
	
	/** The offset between Celsius and Kelvin. */
	public static final long C = 273; // 273.15 IRL
	
	/** The Default Environment Temperature in Kelvin */
	public static final long DEFAULT_ENVIRONMENT_TEMPERATURE = C + 20; // 273.15 IRL
	
	/** The Item WildCard Tag. Even shorter than the "-1" of the past */
	public static final short W = OreDictionary.WILDCARD_VALUE;
	
	/** The Size of an infinite NEI ItemStack */
	public static final byte NEI_INFINITE = 111;
	
	/** The Voltage Tiers. Use this Array instead of the old named Voltage Variables */
	public static final long[] V = new long[] {8, 32, 128, 512, 2048, 8192, 32768, 131072, 524288, Integer.MAX_VALUE, Integer.MAX_VALUE, Integer.MAX_VALUE, Integer.MAX_VALUE, Integer.MAX_VALUE, Integer.MAX_VALUE, Integer.MAX_VALUE};
	
	/** The short Names for the Voltages */
	public static final String[] VN = new String[] {"ULV", "LV", "MV", "HV", "EV", "IV", "LuV", "ZPM", "UV", "MAX", "MAX", "MAX", "MAX", "MAX", "MAX", "MAX"};
	
	/** The long Names for the Voltages */
	public static final String[] VOLTAGE_NAMES = new String[] {"Ultra Low Voltage", "Low Voltage", "Medium Voltage", "High Voltage", "Extreme Voltage", "Insane Voltage", "Ludicrous Voltage", "ZPM Voltage", "Ultimate Voltage", "Maximum Voltage", "Maximum Voltage", "Maximum Voltage", "Maximum Voltage", "Maximum Voltage", "Maximum Voltage", "Maximum Voltage"};
	
	/** The value of how many RF Units are worth an Energy Unit (EU) or MJ Unit. */
	public static final int RF_PER_EU = 4, RF_PER_MJ = 10;
	
	/** The value of how many Energy Units are worth a Smelt. This is for Fuel -> Energy, not for Energy -> Smelt, because many Mods including IC2 have cheaper smelting. In GT, I go for a Max Efficiency of 12.5 times that value, so 400 EU per Furnace Smelt. */
	public static int EU_PER_SMELT = 5000, TICKS_PER_SMELT = 200;
	
	/** The value of how many Energy Units a Litre of Lava is worth. It is worth about 180 to 333 RF in TE. Well, I go for 320 RF in GT. */
	public static int EU_PER_LAVA = 80;
	
	/** A few Default Values for Light Opacity. */
	public static final int LIGHT_OPACITY_NONE = 0, LIGHT_OPACITY_LEAVES = 1, LIGHT_OPACITY_WATER = 3, LIGHT_OPACITY_MAX = 255;
	
	/** a Random generator so I don't need to instantiate a new one all the time. */
	public static final Random RANDOM = new Random();
	
	/** The Colour White as RGB Short Array. */
	public static final short[] UNCOLOURED = new short[] {255, 255, 255, 255};
	/** The Colour White as simple Integer (0x00ffffff). */
	public static final int UNCOLORED = 16777215;
	
	/** Some Colour Arrays */
	public static final short[]
	  CA_WHITE				= new short[] {255, 255, 255, 255}
	, CA_GRAY_192			= new short[] {192, 192, 192, 255}
	, CA_GRAY_128			= new short[] {128, 128, 128, 255}
	, CA_GRAY_64			= new short[] { 64,  64,  64, 255}
	, CA_GRAY_32			= new short[] { 32,  32,  32, 255}
	, CA_RED_255			= new short[] {255,   0,   0, 255}
	, CA_RED_192			= new short[] {192,   0,   0, 255}
	, CA_RED_128			= new short[] {128,   0,   0, 255}
	, CA_RED_64				= new short[] { 64,   0,   0, 255}
	, CA_RED_32				= new short[] { 32,   0,   0, 255}
	, CA_GREEN_255			= new short[] {  0, 255,   0, 255}
	, CA_GREEN_192			= new short[] {  0, 192,   0, 255}
	, CA_GREEN_128			= new short[] {  0, 128,   0, 255}
	, CA_GREEN_64			= new short[] {  0,  64,   0, 255}
	, CA_GREEN_32			= new short[] {  0,  32,   0, 255}
	, CA_BLUE_255			= new short[] {  0,   0, 255, 255}
	, CA_BLUE_192			= new short[] {  0,   0, 192, 255}
	, CA_BLUE_128			= new short[] {  0,   0, 128, 255}
	, CA_BLUE_64			= new short[] {  0,   0,  64, 255}
	, CA_BLUE_32			= new short[] {  0,   0,  32, 255}
	, CA_LIGHT_BLUE_255		= new short[] {128, 128, 255, 255}
	, CA_LIGHT_BLUE_192		= new short[] { 96,  96, 192, 255}
	, CA_LIGHT_BLUE_128		= new short[] { 64,  64, 128, 255}
	, CA_LIGHT_BLUE_64		= new short[] { 32,  32,  64, 255}
	, CA_LIGHT_BLUE_32		= new short[] { 16,  16,  32, 255}
	, CA_YELLOW_255			= new short[] {255, 255,   0, 255}
	, CA_YELLOW_192			= new short[] {192, 192,   0, 255}
	, CA_YELLOW_128			= new short[] {128, 128,   0, 255}
	, CA_YELLOW_64			= new short[] { 64,  64,   0, 255}
	, CA_YELLOW_32			= new short[] { 32,  32,   0, 255}
	, CA_LIGHT_YELLOW_255	= new short[] {255, 255, 128, 255}
	, CA_LIGHT_YELLOW_192	= new short[] {192, 192,  96, 255}
	, CA_LIGHT_YELLOW_128	= new short[] {128, 128,  64, 255}
	, CA_LIGHT_YELLOW_64	= new short[] { 64,  64,  32, 255}
	, CA_LIGHT_YELLOW_32	= new short[] { 32,  32,  16, 255}
	;
	
	public static final short[]
	DYE_None			= {255, 255, 255, 255},
	DYE_Black			= { 32,  32,  32, 255},
	DYE_Red				= {255,   0,   0, 255},
	DYE_Green			= {  0, 255,   0, 255},
	DYE_Brown			= { 96,  64,   0, 255},
	DYE_Blue			= {  0,   0, 255, 255},
	DYE_Purple			= {128,   0, 128, 255},
	DYE_Cyan			= {  0, 255, 255, 255},
	DYE_LightGray		= {192, 192, 192, 255},
	DYE_Gray			= {128, 128, 128, 255},
	DYE_Pink			= {255, 192, 192, 255},
	DYE_Lime			= {128, 255, 128, 255},
	DYE_Yellow			= {255, 255,   0, 255},
	DYE_LightBlue		= {128, 128, 255, 255},
	DYE_Magenta			= {255,   0, 255, 255},
	DYE_Orange			= {255, 128,   0, 255},
	DYE_White			= {255, 255, 255, 255};
	
	public static final int
	DYE_INT_None			= UT.Code.getRGBInt(DYE_None),
	DYE_INT_Black			= UT.Code.getRGBInt(DYE_Black),
	DYE_INT_Red				= UT.Code.getRGBInt(DYE_Red),
	DYE_INT_Green			= UT.Code.getRGBInt(DYE_Green),
	DYE_INT_Brown			= UT.Code.getRGBInt(DYE_Brown),
	DYE_INT_Blue			= UT.Code.getRGBInt(DYE_Blue),
	DYE_INT_Purple			= UT.Code.getRGBInt(DYE_Purple),
	DYE_INT_Cyan			= UT.Code.getRGBInt(DYE_Cyan),
	DYE_INT_LightGray		= UT.Code.getRGBInt(DYE_LightGray),
	DYE_INT_Gray			= UT.Code.getRGBInt(DYE_Gray),
	DYE_INT_Pink			= UT.Code.getRGBInt(DYE_Pink),
	DYE_INT_Lime			= UT.Code.getRGBInt(DYE_Lime),
	DYE_INT_Yellow			= UT.Code.getRGBInt(DYE_Yellow),
	DYE_INT_LightBlue		= UT.Code.getRGBInt(DYE_LightBlue),
	DYE_INT_Magenta			= UT.Code.getRGBInt(DYE_Magenta),
	DYE_INT_Orange			= UT.Code.getRGBInt(DYE_Orange),
	DYE_INT_White			= UT.Code.getRGBInt(DYE_White);
	
	public static final byte
	DYE_INDEX_Black			=  0,
	DYE_INDEX_Red			=  1,
	DYE_INDEX_Green			=  2,
	DYE_INDEX_Brown			=  3,
	DYE_INDEX_Blue			=  4,
	DYE_INDEX_Purple		=  5,
	DYE_INDEX_Cyan			=  6,
	DYE_INDEX_LightGray		=  7,
	DYE_INDEX_Gray			=  8,
	DYE_INDEX_Pink			=  9,
	DYE_INDEX_Lime			= 10,
	DYE_INDEX_Yellow		= 11,
	DYE_INDEX_LightBlue		= 12,
	DYE_INDEX_Magenta		= 13,
	DYE_INDEX_Orange		= 14,
	DYE_INDEX_White			= 15;
	
	public static final String[] DYE_NAMES					= {"Black", "Red", "Green", "Brown", "Blue", "Purple", "Cyan", "Light Gray", "Gray", "Pink", "Lime", "Yellow", "Light Blue", "Magenta", "Orange", "White"};
	public static final String[] DYE_OREDICTS				= {"dyeBlack", "dyeRed", "dyeGreen", "dyeBrown", "dyeBlue", "dyePurple", "dyeCyan", "dyeLightGray", "dyeGray", "dyePink", "dyeLime", "dyeYellow", "dyeLightBlue", "dyeMagenta", "dyeOrange", "dyeWhite"};
	public static final String[] DYE_OREDICTS_MIXABLE		= {"dyeMixableBlack", "dyeMixableRed", "dyeMixableGreen", "dyeMixableBrown", "dyeMixableBlue", "dyeMixablePurple", "dyeMixableCyan", "dyeMixableLightGray", "dyeMixableGray", "dyeMixablePink", "dyeMixableLime", "dyeMixableYellow", "dyeMixableLightBlue", "dyeMixableMagenta", "dyeMixableOrange", "dyeMixableWhite"};
	public static final String[] DYE_OREDICTS_POST			= {"Black", "Red", "Green", "Brown", "Blue", "Purple", "Cyan", "LightGray", "Gray", "Pink", "Lime", "Yellow", "LightBlue", "Magenta", "Orange", "White"};
	
	public static final short[][] DYES						= {DYE_Black, DYE_Red, DYE_Green, DYE_Brown, DYE_Blue, DYE_Purple, DYE_Cyan, DYE_LightGray, DYE_Gray, DYE_Pink, DYE_Lime, DYE_Yellow, DYE_LightBlue, DYE_Magenta, DYE_Orange, DYE_White};
	public static final short[][] DYES_INVERTED				= {DYES[15], DYES[14], DYES[13], DYES[12], DYES[11], DYES[10], DYES[ 9], DYES[ 8], DYES[ 7], DYES[ 6], DYES[ 5], DYES[ 4], DYES[ 3], DYES[ 2], DYES[ 1], DYES[ 0]};
	
	public static final int[] DYES_INT						= {DYE_INT_Black, DYE_INT_Red, DYE_INT_Green, DYE_INT_Brown, DYE_INT_Blue, DYE_INT_Purple, DYE_INT_Cyan, DYE_INT_LightGray, DYE_INT_Gray, DYE_INT_Pink, DYE_INT_Lime, DYE_INT_Yellow, DYE_INT_LightBlue, DYE_INT_Magenta, DYE_INT_Orange, DYE_INT_White};
	public static final int[] DYES_INT_INVERTED				= {DYES_INT[15], DYES_INT[14], DYES_INT[13], DYES_INT[12], DYES_INT[11], DYES_INT[10], DYES_INT[ 9], DYES_INT[ 8], DYES_INT[ 7], DYES_INT[ 6], DYES_INT[ 5], DYES_INT[ 4], DYES_INT[ 3], DYES_INT[ 2], DYES_INT[ 1], DYES_INT[ 0]};
	
	public static final FluidStack[] DYE_FLUIDS_WATER		= new FluidStack[16];
	public static final FluidStack[] DYE_FLUIDS_CHEMICAL	= new FluidStack[16];
	public static final FluidStack[] DYED_C_FOAMS			= new FluidStack[16];
	
	public static final ArrayListNoNulls<FluidStack>[] DYE_FLUIDS = new ArrayListNoNulls[] {new ArrayListNoNulls(), new ArrayListNoNulls(), new ArrayListNoNulls(), new ArrayListNoNulls(), new ArrayListNoNulls(), new ArrayListNoNulls(), new ArrayListNoNulls(), new ArrayListNoNulls(), new ArrayListNoNulls(), new ArrayListNoNulls(), new ArrayListNoNulls(), new ArrayListNoNulls(), new ArrayListNoNulls(), new ArrayListNoNulls(), new ArrayListNoNulls(), new ArrayListNoNulls()};
	
	/** Array with length 17 containing the Coordinates of Pixels from 0/16 to 16/16 */
	public static final float[] PIXELS_POS = new float[] {
		  0.00F, 0.0625F, 0.125F, 0.1875F
		, 0.25F, 0.3125F, 0.375F, 0.4375F
		, 0.50F, 0.5625F, 0.625F, 0.6875F
		, 0.75F, 0.8125F, 0.875F, 0.9375F
		, 1.00F};
	
	/** Array with length 17 containing the Coordinates of Pixels from 16/16 to 0/16 */
	public static final float[] PIXELS_NEG = new float[] {
		  1.00F, 0.9375F, 0.875F, 0.8125F
		, 0.75F, 0.6875F, 0.625F, 0.5625F
		, 0.50F, 0.4375F, 0.375F, 0.3125F
		, 0.25F, 0.1875F, 0.125F, 0.0625F
		, 0.00F};
	
	/** Different Side Variables for easier comprehension. */
	public static final byte		SIDE_Y_NEG	= 0, SIDE_BOTTOM	= 0, SIDE_DOWN		= 0,
									SIDE_Y_POS	= 1, SIDE_TOP		= 1, SIDE_UP		= 1,
									SIDE_Z_NEG	= 2, SIDE_NORTH		= 2, // Also a Side with a stupidly mirrored Texture
									SIDE_Z_POS	= 3, SIDE_SOUTH		= 3,
									SIDE_X_NEG	= 4, SIDE_WEST 		= 4,
									SIDE_X_POS	= 5, SIDE_EAST 		= 5, // Also a Side with a stupidly mirrored Texture
									SIDE_ANY	= 6, SIDE_UNKNOWN	= 6, SIDE_INVALID = 6, SIDE_INSIDE = 6, SIDE_UNDEFINED = 6;
	
	/**
	 * [Facing,Side]->Side Mappings for Blocks, which don't face up- and downwards.
	 * 0 = bottom, 1 = top, 2 = left, 3 = front, 4 = right, 5 = back, 6 = undefined.
	 */
	public static final byte[][] FACING_ROTATIONS = new byte[][] {
		{0,1,2,3,4,5,6},
		{0,1,2,3,4,5,6},
		{0,1,3,5,4,2,6},
		{0,1,5,3,2,4,6},
		{0,1,2,4,3,5,6},
		{0,1,4,2,5,3,6},
		{0,1,2,3,4,5,6}
	};
	
	/**
	 * [Facing,Side]->Orientation Mappings for Blocks, which don't face up- and downwards.
	 * 0 = bottom, 1 = top, 2 = left, 3 = front, 4 = right, 5 = back, 6 = undefined.
	 */
	public static final byte[][] FACING_TO_SIDE = new byte[][] {
		{0,1,2,3,4,5,6},
		{0,1,2,3,4,5,6},
		{0,1,5,2,4,3,6},
		{0,1,4,3,5,2,6},
		{0,1,2,4,3,5,6},
		{0,1,3,5,2,4,6},
		{0,1,2,3,4,5,6}
	};
	
	/** Gives you the Sides, which are not the Front nor the Back of the Facing. */
	public static final byte[][] FACING_SIDES = new byte[][] {{2,3,4,5},{2,3,4,5},{0,1,4,5},{0,1,4,5},{0,1,2,3},{0,1,2,3},{}};
	/** Gives you the Sides, which are the Front and the Back, with the Front being the first one. */
	public static final byte[][] FACING_FRONT_BACK = new byte[][] {{0,1},{1,0},{2,3},{3,2},{4,5},{5,4},{}};
	/** Gives you the Sides, which are the Front and the Back, with the Back being the first one. */
	public static final byte[][] FACING_BACK_FRONT = new byte[][] {{1,0},{0,1},{3,2},{2,3},{5,4},{4,5},{}};
	
	/** Checks if two Sides are along the same Axis */
	public static final boolean[][] ALONG_AXIS = new boolean[][] {
		{T,T,F,F,F,F,F},
		{T,T,F,F,F,F,F},
		{F,F,T,T,F,F,F},
		{F,F,T,T,F,F,F},
		{F,F,F,F,T,T,F},
		{F,F,F,F,T,T,F},
		{F,F,F,F,F,F,T}
	};
	
	/** Checks if two Sides are along the shifted Axis */
	public static final boolean[][] ALONG_AXIS_1 = new boolean[][] {
		{F,F,T,T,F,F,F},
		{F,F,T,T,F,F,F},
		{F,F,F,F,T,T,F},
		{F,F,F,F,T,T,F},
		{T,T,F,F,F,F,F},
		{T,T,F,F,F,F,F},
		{F,F,F,F,F,F,F}
	};
	
	/** Checks if two Sides are along the double shifted Axis */
	public static final boolean[][] ALONG_AXIS_2 = new boolean[][] {
		{F,F,F,F,T,T,F},
		{F,F,F,F,T,T,F},
		{T,T,F,F,F,F,F},
		{T,T,F,F,F,F,F},
		{F,F,T,T,F,F,F},
		{F,F,T,T,F,F,F},
		{F,F,F,F,F,F,F}
	};
	
	/** Insert Facing and a Connectivity BitMask to see if it is connecting to that Side. Technically this is a simple Bit Operation, but accessing an Array with "FACE_CONNECTED[aSide][aConnections]" just looks nicer than "(aConnections & (1 << aSide) != 0)". */
	public static final boolean[][] FACE_CONNECTED = new boolean[][] {
		{F,T,F,T,F,T,F,T,F,T,F,T,F,T,F,T,F,T,F,T,F,T,F,T,F,T,F,T,F,T,F,T,F,T,F,T,F,T,F,T,F,T,F,T,F,T,F,T,F,T,F,T,F,T,F,T,F,T,F,T,F,T,F,T,F,T,F,T,F,T,F,T,F,T,F,T,F,T,F,T,F,T,F,T,F,T,F,T,F,T,F,T,F,T,F,T,F,T,F,T,F,T,F,T,F,T,F,T,F,T,F,T,F,T,F,T,F,T,F,T,F,T,F,T,F,T,F,T},
		{F,F,T,T,F,F,T,T,F,F,T,T,F,F,T,T,F,F,T,T,F,F,T,T,F,F,T,T,F,F,T,T,F,F,T,T,F,F,T,T,F,F,T,T,F,F,T,T,F,F,T,T,F,F,T,T,F,F,T,T,F,F,T,T,F,F,T,T,F,F,T,T,F,F,T,T,F,F,T,T,F,F,T,T,F,F,T,T,F,F,T,T,F,F,T,T,F,F,T,T,F,F,T,T,F,F,T,T,F,F,T,T,F,F,T,T,F,F,T,T,F,F,T,T,F,F,T,T},
		{F,F,F,F,T,T,T,T,F,F,F,F,T,T,T,T,F,F,F,F,T,T,T,T,F,F,F,F,T,T,T,T,F,F,F,F,T,T,T,T,F,F,F,F,T,T,T,T,F,F,F,F,T,T,T,T,F,F,F,F,T,T,T,T,F,F,F,F,T,T,T,T,F,F,F,F,T,T,T,T,F,F,F,F,T,T,T,T,F,F,F,F,T,T,T,T,F,F,F,F,T,T,T,T,F,F,F,F,T,T,T,T,F,F,F,F,T,T,T,T,F,F,F,F,T,T,T,T},
		{F,F,F,F,F,F,F,F,T,T,T,T,T,T,T,T,F,F,F,F,F,F,F,F,T,T,T,T,T,T,T,T,F,F,F,F,F,F,F,F,T,T,T,T,T,T,T,T,F,F,F,F,F,F,F,F,T,T,T,T,T,T,T,T,F,F,F,F,F,F,F,F,T,T,T,T,T,T,T,T,F,F,F,F,F,F,F,F,T,T,T,T,T,T,T,T,F,F,F,F,F,F,F,F,T,T,T,T,T,T,T,T,F,F,F,F,F,F,F,F,T,T,T,T,T,T,T,T},
		{F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T},
		{F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T},
		{F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,F,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T,T}
	};
	
	/** Fast lookup to see how many Connections a Mask has. It is recommended to do either &63 or &127 on the Index depending on how you use it. */
	public static final byte[]				FACE_CONNECTION_COUNT = new byte[] {0,1,1,2,1,2,2,3,1,2,2,3,2,3,3,4,1,2,2,3,2,3,3,4,2,3,3,4,3,4,4,5,1,2,2,3,2,3,3,4,2,3,3,4,3,4,4,5,2,3,3,4,3,4,4,5,3,4,4,5,4,5,5,6,1,2,2,3,2,3,3,4,2,3,3,4,3,4,4,5,2,3,3,4,3,4,4,5,3,4,4,5,4,5,5,6,2,3,3,4,3,4,4,5,3,4,4,5,4,5,5,6,3,4,4,5,4,5,5,6,4,5,5,6,5,6,6,7};
	
	public static final byte[]				SIDE_BITS = new byte[] { 1, 2, 4, 8,16,32,64};
	
	/** Those are not representing actual directions! They are for the "FACING_ROTATIONS" Array-Map */
	public static final byte				SIDE_LEFT = 2, SIDE_FRONT = 3, SIDE_RIGHT = 4, SIDE_BACK = 5;
	
	/** Converts Sides to a Top-Bottom-Side Value, this limits the Range to a Number between [0 and 2] */
	public static final byte[]				FACES_TBS = new byte[] { 0, 1, 2, 2, 2, 2, 2};
	/** Side->Opposite Mappings. */
	public static final byte[]				OPPOSITES = new byte[] { 1, 0, 3, 2, 5, 4, 6};
	/** Side->Offset Mappings. */
	public static final byte[]				OFFSETS_X = new byte[] { 0, 0, 0, 0,-1,+1, 0},
											OFFSETS_Y = new byte[] {-1,+1, 0, 0, 0, 0, 0},
											OFFSETS_Z = new byte[] { 0, 0,-1,+1, 0, 0, 0};
	
	/** Side->ForgeDirection Mappings. */
	public static final ForgeDirection[]	FORGE_DIR = new ForgeDirection[] {ForgeDirection.DOWN, ForgeDirection.UP, ForgeDirection.NORTH, ForgeDirection.SOUTH, ForgeDirection.WEST, ForgeDirection.EAST, ForgeDirection.UNKNOWN};
	/** Side->Opposite Mappings with ForgeDirection. */
	public static final ForgeDirection[]	FORGE_DIR_OPPOSITES = new ForgeDirection[] {ForgeDirection.UP, ForgeDirection.DOWN, ForgeDirection.SOUTH, ForgeDirection.NORTH, ForgeDirection.EAST, ForgeDirection.WEST, ForgeDirection.UNKNOWN};
	
	/** Compass alike Array for the proper ordering of North, East, South and West. */
	public static final byte[]				COMPASS_DIRECTIONS		= new byte[] {SIDE_NORTH, SIDE_EAST, SIDE_SOUTH, SIDE_WEST};
	/** Side -> Compass Direction. Defaults to North if wrong value. */
	public static final byte[]				COMPASS_FROM_SIDE		= new byte[] { 0, 0, 0, 2, 3, 1, 0};
	
	/** An Array containing all Sides which follow the Condition, in order to iterate over them for example. */
	public static final byte[]				ALL_SIDES					= new byte[]    { 0, 1, 2, 3, 4, 5, 6},
											ALL_SIDES_MIDDLE			= new byte[]    { 6, 0, 1, 2, 3, 4, 5},
											ALL_SIDES_MIDDLE_UP			= new byte[]    { 6, 1, 2, 3, 4, 5, 0},
											ALL_SIDES_MIDDLE_DOWN		= new byte[]    { 6, 0, 2, 3, 4, 5, 1},
											ALL_SIDES_VALID				= new byte[]    { 0, 1, 2, 3, 4, 5},
											ALL_SIDES_VALID_FIRST[]		= new byte[][] {{ 0, 1, 2, 3, 4, 5}, { 1, 0, 2, 3, 4, 5}, { 2, 0, 1, 3, 4, 5}, { 3, 0, 1, 2, 4, 5}, { 4, 0, 1, 2, 3, 5}, { 5, 0, 1, 2, 3, 4}, { 0, 1, 2, 3, 4, 5}},
											ALL_SIDES_VALID_BUT[]		= new byte[][] {{ 1, 2, 3, 4, 5}, { 0, 2, 3, 4, 5}, { 0, 1, 3, 4, 5}, { 0, 1, 2, 4, 5}, { 0, 1, 2, 3, 5}, { 0, 1, 2, 3, 4}, { 0, 1, 2, 3, 4, 5}},
											ALL_SIDES_VERTICAL			= new byte[]    { 0, 1},
											ALL_SIDES_HORIZONTAL		= new byte[]    { 2, 3, 4, 5},
											ALL_SIDES_HORIZONTAL_UP		= new byte[]    { 2, 3, 4, 5, 1},
											ALL_SIDES_HORIZONTAL_DOWN	= new byte[]    { 2, 3, 4, 5, 0},
											ALL_SIDES_BUT_TOP			= new byte[]    { 0, 2, 3, 4, 5},
											ALL_SIDES_BUT_BOTTOM		= new byte[]    { 1, 2, 3, 4, 5},
											ALL_SIDES_X					= new byte[]    { 4, 5},
											ALL_SIDES_Y					= new byte[]    { 0, 1},
											ALL_SIDES_Z					= new byte[]    { 2, 3};
	
	/** For Facing Checks. */
	public static final boolean[]			SIDES_BOTTOM			= new boolean[] {T,F,F,F,F,F,F},
											SIDES_TOP				= new boolean[] {F,T,F,F,F,F,F},
											SIDES_LEFT				= new boolean[] {F,F,T,F,F,F,F},
											SIDES_FRONT				= new boolean[] {F,F,F,T,F,F,F},
											SIDES_RIGHT				= new boolean[] {F,F,F,F,T,F,F},
											SIDES_BACK				= new boolean[] {F,F,F,F,F,T,F},
											SIDES_INVALID			= new boolean[] {F,F,F,F,F,F,T},
											SIDES_VALID				= new boolean[] {T,T,T,T,T,T,F},
											SIDES_ALL				= new boolean[] {T,T,T,T,T,T,T},
											SIDES_NONE				= new boolean[] {F,F,F,F,F,F,F},
											SIDES_LEFT_RIGHT		= new boolean[] {F,F,T,F,T,F,F},
											SIDES_FRONT_BACK		= new boolean[] {F,F,F,T,F,T,F},
											SIDES_AXIS_X			= new boolean[] {F,F,F,F,T,T,F},
											SIDES_AXIS_Y			= new boolean[] {T,T,F,F,F,F,F},
											SIDES_AXIS_Z			= new boolean[] {F,F,T,T,F,F,F},
											SIDES_COMPASS			= new boolean[] {F,F,T,T,T,T,F},
											SIDES_VERTICAL			= new boolean[] {T,T,F,F,F,F,F},
											SIDES_HORIZONTAL		= new boolean[] {F,F,T,T,T,T,F},
											SIDES_TOP_HORIZONTAL	= new boolean[] {F,T,T,T,T,T,F},
											SIDES_BOTTOM_HORIZONTAL	= new boolean[] {T,F,T,T,T,T,F};
	
	/** For Facing Checks. */
	public static final boolean[][]
	SIDES_ANY_BUT = new boolean[][] {
		{F,T,T,T,T,T,F},
		{T,F,T,T,T,T,F},
		{T,T,F,T,T,T,F},
		{T,T,T,F,T,T,F},
		{T,T,T,T,F,T,F},
		{T,T,T,T,T,F,F},
		{T,T,T,T,T,T,F}
	},
	SIDES_THIS = new boolean[][] {
		{T,F,F,F,F,F,F},
		{F,T,F,F,F,F,F},
		{F,F,T,F,F,F,F},
		{F,F,F,T,F,F,F},
		{F,F,F,F,T,F,F},
		{F,F,F,F,F,T,F},
		{F,F,F,F,F,F,F}
	};
	
	public static final boolean[] TRUE_6 = new boolean[] {T,T,T,T,T,T};
	
	/** Zero-Length Array to save on Memory. */ public static final byte					[] ZL_BYTE				= new byte[0];
	/** Zero-Length Array to save on Memory. */ public static final short					[] ZL_SHORT				= new short[0];
	/** Zero-Length Array to save on Memory. */ public static final int						[] ZL_INTEGER			= new int[0];
	/** Zero-Length Array to save on Memory. */ public static final long					[] ZL_LONG				= new long[0];
	/** Zero-Length Array to save on Memory. */ public static final float					[] ZL_FLOAT				= new float[0];
	/** Zero-Length Array to save on Memory. */ public static final double					[] ZL_DOUBLE			= new double[0];
	/** Zero-Length Array to save on Memory. */ public static final Object					[] ZL_OBJECT			= new Object[0];
	/** Zero-Length Array to save on Memory. */ public static final String					[] ZL_STRING			= new String[0];
	/** Zero-Length Array to save on Memory. */ public static final TagData					[] ZL_TAGDATA			= new TagData[0];
	/** Zero-Length Array to save on Memory. */ public static final ItemStack				[] ZL_ITEMSTACK			= new ItemStack[0];
	/** Zero-Length Array to save on Memory. */ public static final FluidStack				[] ZL_FLUIDSTACK		= new FluidStack[0];
	/** Zero-Length Array to save on Memory. */ public static final FluidTank				[] ZL_FLUIDTANK			= new FluidTank[0];
	/** Zero-Length Array to save on Memory. */ public static final IFluidTank				[] ZL_IFLUIDTANK		= new IFluidTank[0];
	/** Zero-Length Array to save on Memory. */ public static final FluidTankInfo			[] ZL_FLUIDTANKINFO		= new FluidTankInfo[0];
	/** Zero-Length Array to save on Memory. */ public static final OreDictMaterial			[] ZL_MATERIAL			= new OreDictMaterial[0];
	/** Zero-Length Array to save on Memory. */ public static final OreDictMaterialStack	[] ZL_MATERIALSTACK		= new OreDictMaterialStack[0];
	/** Zero-Length Array to save on Memory. */ public static final OreDictItemData			[] ZL_OREDICTITEMDATA	= new OreDictItemData[0];
	/** Zero-Length Array to save on Memory. */ public static final ForgeDirection			[] ZL_FORGEDIRECTION	= new ForgeDirection[0];
	/** Zero-Length Array to save on Memory. */ public static final Recipe					[] ZL_RECIPE			= new Recipe[0];
	/** Zero-Length Array to save on Memory. */ public static final IIconContainer			[] ZL_IICONCONTAINER	= new IIconContainer[0], L6_IICONCONTAINER	= new IIconContainer[6];
	
	/** This way it is possible to have a Call Hierarchy of NullPointers in ItemStack based Functions, and also because most of the time I don't know what kind of Data Type the "null" stands for, when there are shitloads of Parameters for a Function */
	public static final ItemStack NI = null;
	
	/** This way it is possible to have a Call Hierarchy of NullPointers in FluidStack based Functions, and also because most of the time I don't know what kind of Data Type the "null" stands for, when there are shitloads of Parameters for a Function */
	public static final FluidStack NF = null;
	
	/** This way it is possible to have a Call Hierarchy of NullPointers in Block based Functions, and also because most of the time I don't know what kind of Data Type the "null" stands for, when there are shitloads of Parameters for a Function */
	public static final Block NB = Blocks.air;
	
	/** States of Matter */
	public static final byte STATE_SOLID = 0, STATE_LIQUID = 1, STATE_GASEOUS = 2, STATE_PLASMA = 3;
	
	/** The weight of Air at Atmospheric Pressure per Cubic Centimetre. */
	public static final double WEIGHT_AIR_G_PER_CUBIC_CENTIMETER = 0.0012;
	/** The weight of Air at Atmospheric Pressure per Cubic Meter. */
	public static final double WEIGHT_AIR_KG_PER_CUBIC_METER = 1.2;
	/** The weight of Air at Atmospheric Pressure per Material Unit. */
	public static final double WEIGHT_AIR_KG_PER_UNIT = WEIGHT_AIR_KG_PER_CUBIC_METER / 9;
	
	/** Not really Constants, but they set using the Config and therefore should be constant. */
	public static boolean D1 = F, D2 = F, D3 = F, NEI = F, NERFED_WOOD = F, OVERCHARGE_EXPLOSIONS = F, SHOW_HIDDEN_MATERIALS = F, SHOW_HIDDEN_PREFIXES = F, SHOW_ORE_BLOCK_PREFIXES = F, SHOW_HIDDEN_ITEMS = F, HUNGER_BY_INVENTORY_WEIGT = F, INVENTORY_UNIFICATION = T, ADVENTURE_MODE_AXE = T, SURVIVAL_INTO_ADVENTURE_MODE = F;
	/** This means that Client or Server specific Base Files are definitely existing and loaded! Not if the World is actually client side or server side! */
	public static boolean CODE_CLIENT = F, CODE_SERVER = F;
	
	/** Not really Constants, but they set using the Config and therefore should be constant. */
	public static double HARDNESS_MULTIPLIER_SAND = 1.0, HARDNESS_MULTIPLIER_ROCK = 1.0, HARDNESS_MULTIPLIER_ORES = 1.0;
	/** Those are the values derived directly by the Configuration File. DO NOT USE THEM, USE THE VALUES ABOVE INSTEAD!!! */
	public static double CONFIG_HARDNESS_MULTIPLIER_SAND = 1.0, CONFIG_HARDNESS_MULTIPLIER_ROCK = 1.0, CONFIG_HARDNESS_MULTIPLIER_ORES = 1.0;
	
	/** If you have to give something a World Parameter but there is no World... (Dummy World) */
	public static World DW;
	
	/** The Object containing the actual Mod GregTech and its API. */
	public static Abstract_Mod GT, GAPI, GAPI_POST;
	
	/** Don't use this Object to add Recipes anymore, just go into the RecipeMap Class and add the Recipes there. (old Recipe Adder, going to be removed) */
	@Deprecated public static IGT_RecipeAdder RA = new NonWorkingRecipeAdder();
	
	/** For Internal Usage. */
	public static INetworkHandler NW_GT;
	
	/** For using the API internal Network Functionality for the already existing Packets. */
	public static INetworkHandler NW_API;
	
	/** Date and Time of when the Game launched. */
	public static final String DATE_OF_GAME_START = UT.Code.dateAndTime();
	
	/** A Set of different Tool Names. */
	public static final String
	  TOOL_LOCALISER_PREFIX	= "gt.lang.tool.name."
	, TOOL_TOOLTIP_PREFIX	= "gt.lang.tool.tooltip."
	, TOOL_rotator			= "rotator"
	, TOOL_igniter			= "igniter"
	, TOOL_extinguisher		= "extinguisher"
	, TOOL_whacker			= "whacker"
	, TOOL_magnifyingglass	= "magnifyingglass"
	, TOOL_wrench			= "wrench"
	, TOOL_monkeywrench		= "monkeywrench"
	, TOOL_crowbar			= "crowbar"
	, TOOL_pincers			= "pincers"
	, TOOL_axe				= "axe"
	, TOOL_pickaxe			= "pickaxe"
	, TOOL_sword			= "sword"
	, TOOL_shovel			= "shovel"
	, TOOL_hoe				= "hoe"
	, TOOL_grafter			= "grafter"
	, TOOL_saw				= "saw"
	, TOOL_file				= "file"
	, TOOL_hammer			= "hammer"
	, TOOL_plow				= "plow"
	, TOOL_plunger			= "plunger"
	, TOOL_scoop			= "scoop"
	, TOOL_shears			= "shears"
	, TOOL_scissors			= "scissors"
	, TOOL_screwdriver		= "screwdriver"
	, TOOL_chisel			= "chisel"
	, TOOL_sense			= "sense"
	, TOOL_scythe			= "scythe"
	, TOOL_softhammer		= "softhammer"
	, TOOL_cutter			= "cutter"
	, TOOL_plasmatorch		= "plasmatorch"
	, TOOL_solderingtool	= "solderingtool"
	, TOOL_solderingmetal	= "solderingmetal"
	, TOOL_thermometer		= "thermometer"
	, TOOL_prospector		= "prospector"
	;
	
	static {
		LH.add(TOOL_LOCALISER_PREFIX + TOOL_rotator			, "Rotation Tool");
		LH.add(TOOL_LOCALISER_PREFIX + TOOL_igniter			, "Igniting Tool");
		LH.add(TOOL_LOCALISER_PREFIX + TOOL_extinguisher	, "Extinguishing Tool");
		LH.add(TOOL_LOCALISER_PREFIX + TOOL_whacker			, "Whacking Tool");
		LH.add(TOOL_LOCALISER_PREFIX + TOOL_wrench			, "Wrench");
		LH.add(TOOL_LOCALISER_PREFIX + TOOL_monkeywrench	, "Monkey Wrench");
		LH.add(TOOL_LOCALISER_PREFIX + TOOL_magnifyingglass	, "Magnifying Glass");
		LH.add(TOOL_LOCALISER_PREFIX + TOOL_crowbar			, "Crowbar");
		LH.add(TOOL_LOCALISER_PREFIX + TOOL_pincers			, "Pincers");
		LH.add(TOOL_LOCALISER_PREFIX + TOOL_axe				, "Axe");
		LH.add(TOOL_LOCALISER_PREFIX + TOOL_pickaxe			, "Pickaxe");
		LH.add(TOOL_LOCALISER_PREFIX + TOOL_sword			, "Sword");
		LH.add(TOOL_LOCALISER_PREFIX + TOOL_shovel			, "Shovel");
		LH.add(TOOL_LOCALISER_PREFIX + TOOL_hoe				, "Hoe");
		LH.add(TOOL_LOCALISER_PREFIX + TOOL_grafter			, "Grafter");
		LH.add(TOOL_LOCALISER_PREFIX + TOOL_saw				, "Saw");
		LH.add(TOOL_LOCALISER_PREFIX + TOOL_file			, "File");
		LH.add(TOOL_LOCALISER_PREFIX + TOOL_hammer			, "Hammer");
		LH.add(TOOL_LOCALISER_PREFIX + TOOL_plow			, "Plow");
		LH.add(TOOL_LOCALISER_PREFIX + TOOL_plunger			, "Plunger");
		LH.add(TOOL_LOCALISER_PREFIX + TOOL_scoop			, "Scoop");
		LH.add(TOOL_LOCALISER_PREFIX + TOOL_shears			, "Shears");
		LH.add(TOOL_LOCALISER_PREFIX + TOOL_scissors		, "Scissors");
		LH.add(TOOL_LOCALISER_PREFIX + TOOL_screwdriver		, "Screwdriver");
		LH.add(TOOL_LOCALISER_PREFIX + TOOL_chisel			, "Chisel");
		LH.add(TOOL_LOCALISER_PREFIX + TOOL_sense			, "Sense");
		LH.add(TOOL_LOCALISER_PREFIX + TOOL_scythe			, "Scythe");
		LH.add(TOOL_LOCALISER_PREFIX + TOOL_softhammer		, "Soft Hammer");
		LH.add(TOOL_LOCALISER_PREFIX + TOOL_cutter			, "Cutter");
		LH.add(TOOL_LOCALISER_PREFIX + TOOL_plasmatorch		, "Plasma Torch");
		LH.add(TOOL_LOCALISER_PREFIX + TOOL_solderingtool	, "Soldering Tool");
		LH.add(TOOL_LOCALISER_PREFIX + TOOL_solderingmetal	, "Soldering Metal");
		LH.add(TOOL_LOCALISER_PREFIX + TOOL_thermometer		, "Thermometer");
		LH.add(TOOL_LOCALISER_PREFIX + TOOL_prospector		, "Prospector");
		
		LH.add(TOOL_TOOLTIP_PREFIX + TOOL_rotator			, "Rotating things");
		LH.add(TOOL_TOOLTIP_PREFIX + TOOL_igniter			, "Igniting things");
		LH.add(TOOL_TOOLTIP_PREFIX + TOOL_extinguisher		, "Extinguishing things");
		LH.add(TOOL_TOOLTIP_PREFIX + TOOL_whacker			, "Whacking things");
		LH.add(TOOL_TOOLTIP_PREFIX + TOOL_wrench			, "Rotates Blocks on Rightclick, dismantles IC2 Blocks");
		LH.add(TOOL_TOOLTIP_PREFIX + TOOL_monkeywrench		, "A special Wrench for secondary Facings and Pipes");
		LH.add(TOOL_TOOLTIP_PREFIX + TOOL_magnifyingglass	, "Used to investigate the Scene");
		LH.add(TOOL_TOOLTIP_PREFIX + TOOL_crowbar			, "Dismounts Covers and Rotates Rails");
		LH.add(TOOL_TOOLTIP_PREFIX + TOOL_pincers			, "Used to grab things without touching them");
		LH.add(TOOL_TOOLTIP_PREFIX + TOOL_axe				, "");
		LH.add(TOOL_TOOLTIP_PREFIX + TOOL_pickaxe			, "");
		LH.add(TOOL_TOOLTIP_PREFIX + TOOL_sword				, "");
		LH.add(TOOL_TOOLTIP_PREFIX + TOOL_shovel			, "");
		LH.add(TOOL_TOOLTIP_PREFIX + TOOL_hoe				, "Can till Dirt and similar Earths");
		LH.add(TOOL_TOOLTIP_PREFIX + TOOL_grafter			, "");
		LH.add(TOOL_TOOLTIP_PREFIX + TOOL_saw				, "");
		LH.add(TOOL_TOOLTIP_PREFIX + TOOL_file				, "");
		LH.add(TOOL_TOOLTIP_PREFIX + TOOL_hammer			, "Can hammer diverse things");
		LH.add(TOOL_TOOLTIP_PREFIX + TOOL_plow				, "");
		LH.add(TOOL_TOOLTIP_PREFIX + TOOL_plunger			, "Emptying clogged Pipes");
		LH.add(TOOL_TOOLTIP_PREFIX + TOOL_scoop				, "");
		LH.add(TOOL_TOOLTIP_PREFIX + TOOL_shears			, "");
		LH.add(TOOL_TOOLTIP_PREFIX + TOOL_scissors			, "");
		LH.add(TOOL_TOOLTIP_PREFIX + TOOL_screwdriver		, "Adjusts Covers, Machines and Redstone Diodes");
		LH.add(TOOL_TOOLTIP_PREFIX + TOOL_chisel			, "Chisels Storage Blocks and chiselable Blocks");
		LH.add(TOOL_TOOLTIP_PREFIX + TOOL_sense				, "Used to mass harvest Crops");
		LH.add(TOOL_TOOLTIP_PREFIX + TOOL_scythe			, "Used to mass harvest Crops");
		LH.add(TOOL_TOOLTIP_PREFIX + TOOL_softhammer		, "Toggles the States of Machines");
		LH.add(TOOL_TOOLTIP_PREFIX + TOOL_cutter			, "Cuts and Connects Wires");
		LH.add(TOOL_TOOLTIP_PREFIX + TOOL_plasmatorch		, "");
		LH.add(TOOL_TOOLTIP_PREFIX + TOOL_solderingtool		, "");
		LH.add(TOOL_TOOLTIP_PREFIX + TOOL_solderingmetal	, "");
		LH.add(TOOL_TOOLTIP_PREFIX + TOOL_thermometer		, "Measuring Temperature");
		LH.add(TOOL_TOOLTIP_PREFIX + TOOL_prospector		, "Prospecting for Ores in an Area");
	}
	
	/** A Set of different NBT Keys I use for Stuff. */
	public static final String
	  NBT_COLOR						= "gt.color"					// Integer
	, NBT_PAINTED					= "gt.painted"					// Boolean
	, NBT_TEXTURE					= "gt.texture"					// String
	, NBT_GUI						= "gt.gui"						// String
	, NBT_MODE						= "gt.mode"						// Number
	, NBT_STATE						= "gt.state"					// Number
	, NBT_MINENERGY					= "gt.minenergy"				// Number
	, NBT_PROGRESS					= "gt.progress"					// Number
	, NBT_MAXPROGRESS				= "gt.maxprogress"				// Number
	, NBT_QUALITY					= "gt.quality"					// Byte
	, NBT_FACING					= "gt.facing"					// Byte
	, NBT_CONNECTION				= "gt.connection"				// Byte
	, NBT_DIAMETER					= "gt.diameter"					// Double
	, NBT_PIPELOSS					= "gt.pipeloss"					// Long
	, NBT_PIPERANGE					= "gt.piperange"				// Long
	, NBT_PIPEBANDWIDTH				= "gt.pipebandwidth"			// Long
	, NBT_PIPESIZE					= "gt.pipesize"					// Long
	, NBT_PIPERENDER				= "gt.piperender"				// Byte
	, NBT_GASPROOF					= "gt.gasproof"					// Boolean
	, NBT_CONTACTDAMAGE				= "gt.contactdamage"			// Boolean
	, NBT_TRANSPARENT				= "gt.transparent"				// Boolean
	, NBT_FOAMED					= "gt.foamed"					// Boolean
	, NBT_FOAMDRIED					= "gt.foamdried"				// Boolean
	, NBT_VISUAL					= "gt.visual"					// Number or Boolean
	, NBT_PISTON					= "gt.piston"					// Number
	, NBT_VALUE						= "gt.value"					// Number
	, NBT_ACTIVE					= "gt.active"					// Boolean containing the active State of a Block.
	, NBT_ACTIVE_ENERGY				= "gt.active.energy"			// Boolean
	, NBT_HARDNESS					= "gt.hardness"					// Float
	, NBT_RESISTANCE				= "gt.resistance"				// Float
	, NBT_REDSTONE					= "gt.redstone"					// Byte
	, NBT_REDSTONE_0				= "gt.redstone.0"				// Byte
	, NBT_REDSTONE_1				= "gt.redstone.1"				// Byte
	, NBT_REDSTONE_2				= "gt.redstone.2"				// Byte
	, NBT_REDSTONE_3				= "gt.redstone.3"				// Byte
	, NBT_REDSTONE_4				= "gt.redstone.4"				// Byte
	, NBT_REDSTONE_5				= "gt.redstone.5"				// Byte
	, NBT_STOPPED					= "gt.stopped"					// Boolean signalising if the Machine is set to stop.
	, NBT_RUNNING					= "gt.running"					// Boolean signalising if the Machine is set up to run.
	, NBT_EFFICIENCY				= "gt.eff"						// Short from 0 to 10000 describing the Efficiency of a Generator or Converter.
	, NBT_INV_SIZE					= "gt.invsize"					// Short
	, NBT_INV_LIST					= "gt.invlist"					// NBT List
	, NBT_INV_OUTPUT				= "gt.invout"					// NBT containing an Output Item. usually postfixed with ".i", 'i' being the index.
	, NBT_INV_SIDE_INPUT			= "gt.invsidein"				// Byte
	, NBT_INV_SIDE_OUTPUT			= "gt.invsideout"				// Byte
	, NBT_INV_SIDE_AUTO_INPUT		= "gt.invsideautoin"			// Byte
	, NBT_INV_SIDE_AUTO_OUTPUT		= "gt.invsideautoout"			// Byte
	, NBT_INV_DISABLED_INPUT		= "gt.invdisabledin"			// Boolean
	, NBT_INV_DISABLED_OUTPUT		= "gt.invdisabledout"			// Boolean
	, NBT_TANK						= "gt.tank"						// NBT containing a Tank. usually postfixed with ".i", 'i' being the index.
	, NBT_TANK_OUTPUT				= "gt.tankout"					// NBT containing an Output Fluid. usually postfixed with ".i", 'i' being the index.
	, NBT_TANK_CAPACITY				= "gt.tankcap"					// Number
	, NBT_TANK_SIDE_INPUT			= "gt.tanksidein"				// Byte
	, NBT_TANK_SIDE_OUTPUT			= "gt.tanksideout"				// Byte
	, NBT_TANK_SIDE_AUTO_INPUT		= "gt.tanksideautoin"			// Byte
	, NBT_TANK_SIDE_AUTO_OUTPUT		= "gt.tanksideautoout"			// Byte
	, NBT_TANK_DISABLED_INPUT		= "gt.tankdisabledin"			// Boolean
	, NBT_TANK_DISABLED_OUTPUT		= "gt.tankdisabledout"			// Boolean
	, NBT_COVERS					= "gt.covers"					// Tag Compound with 24 Tags.
	, NBT_RECIPEMAP					= "gt.recipemap"				// String containing the Recipe Map Name.
	, NBT_FUELMAP					= "gt.fuelmap"					// String containing the Fuel Map Name.
	, NBT_TEMPERATURE				= "gt.temperature"				// Long containing a generic Temperature Variable.
	, NBT_MTE_REG					= "gt.mte.reg"					// Containing the MTE Registry ID
	, NBT_MTE_ID					= "gt.mte.id"					// Containing the MTE ID
	
	, NBT_RECYCLING_COMPS			= "gt.recycling.comps"			// List of Components for the Disassembler.
	, NBT_RECYCLING_MATS			= "gt.recycling.mats"			// List of Materials this Item recycles into. OM.data(ItemStack aStack) will check for this NBT, but only AFTER the Server already started!
	, NBT_FUEL_VALUE				= "gt.fuelvalue"				// Short
	
	, NBT_MATERIAL					= "gt.material"					// String containing the Material Name.
	, NBT_MATERIALS					= "gt.materials"				// Containing the Data of an Array of Material Stacks.
	
	, NBT_ENERGY_EMITTED			= "gt.energy.emitted"			// String containing the Type of emitted Energy.
	, NBT_ENERGY_EMITTED_SIDES		= "gt.energy.emitted.sides"		// String containing the Type of emitted Energy.
	, NBT_ENERGY_EMITTED_TOP		= "gt.energy.emitted.top"		// String containing the Type of emitted Energy.
	, NBT_ENERGY_EMITTED_BOTTOM		= "gt.energy.emitted.bottom"	// String containing the Type of emitted Energy.
	, NBT_ENERGY_EMITTED_FRONT		= "gt.energy.emitted.front"		// String containing the Type of emitted Energy.
	, NBT_ENERGY_EMITTED_BACK		= "gt.energy.emitted.back"		// String containing the Type of emitted Energy.
	, NBT_ENERGY_EMITTED_LEFT		= "gt.energy.emitted.left"		// String containing the Type of emitted Energy.
	, NBT_ENERGY_EMITTED_RIGHT		= "gt.energy.emitted.right"		// String containing the Type of emitted Energy.
	
	, NBT_ENERGY_ACCEPTED			= "gt.energy.accepted"			// String containing the Type of accepted Energy.
	, NBT_ENERGY_ACCEPTED_SIDES		= "gt.energy.accepted.sides"	// String containing the Type of accepted Energy.
	, NBT_ENERGY_ACCEPTED_TOP		= "gt.energy.accepted.top"		// String containing the Type of accepted Energy.
	, NBT_ENERGY_ACCEPTED_BOTTOM	= "gt.energy.accepted.bottom"	// String containing the Type of accepted Energy.
	, NBT_ENERGY_ACCEPTED_FRONT		= "gt.energy.accepted.front"	// String containing the Type of accepted Energy.
	, NBT_ENERGY_ACCEPTED_BACK		= "gt.energy.accepted.back"		// String containing the Type of accepted Energy.
	, NBT_ENERGY_ACCEPTED_LEFT		= "gt.energy.accepted.left"		// String containing the Type of accepted Energy.
	, NBT_ENERGY_ACCEPTED_RIGHT		= "gt.energy.accepted.right"	// String containing the Type of accepted Energy.
	
	, NBT_INPUT						= "gt.input"					// Long containing a generic Energy Variable.
	, NBT_INPUT_MIN					= "gt.input.min"				// Long containing a generic Energy Variable.
	, NBT_INPUT_MAX					= "gt.input.max"				// Long containing a generic Energy Variable.
	, NBT_INPUT_EU					= "gt.input.eu"					// Long containing a specified Energy Variable.
	, NBT_INPUT_RU					= "gt.input.ru"					// Long containing a specified Energy Variable.
	, NBT_INPUT_KU					= "gt.input.ku"					// Long containing a specified Energy Variable.
	, NBT_INPUT_HU					= "gt.input.hu"					// Long containing a specified Energy Variable.
	, NBT_INPUT_LU					= "gt.input.lu"					// Long containing a specified Energy Variable.
	, NBT_INPUT_SU					= "gt.input.su"					// Long containing a specified Energy Variable.
	, NBT_INPUT_AU					= "gt.input.au"					// Long containing a specified Energy Variable.
	, NBT_INPUT_MJ					= "gt.input.mj"					// Long containing a specified Energy Variable.
	, NBT_INPUT_RF					= "gt.input.rf"					// Long containing a specified Energy Variable.
	
	, NBT_OUTPUT					= "gt.output"					// Long containing a generic Energy Variable.
	, NBT_OUTPUT_MIN				= "gt.output.min"				// Long containing a generic Energy Variable.
	, NBT_OUTPUT_MAX				= "gt.output.max"				// Long containing a generic Energy Variable.
	, NBT_OUTPUT_EU					= "gt.output.eu"				// Long containing a specified Energy Variable.
	, NBT_OUTPUT_RU					= "gt.output.ru"				// Long containing a specified Energy Variable.
	, NBT_OUTPUT_KU					= "gt.output.ku"				// Long containing a specified Energy Variable.
	, NBT_OUTPUT_HU					= "gt.output.hu"				// Long containing a specified Energy Variable.
	, NBT_OUTPUT_LU					= "gt.output.lu"				// Long containing a specified Energy Variable.
	, NBT_OUTPUT_SU					= "gt.output.su"				// Long containing a specified Energy Variable.
	, NBT_OUTPUT_AU					= "gt.output.au"				// Long containing a specified Energy Variable.
	, NBT_OUTPUT_MJ					= "gt.output.mj"				// Long containing a specified Energy Variable.
	, NBT_OUTPUT_RF					= "gt.output.rf"				// Long containing a specified Energy Variable.
	
	, NBT_ENERGY					= "gt.energy"					// Long containing a generic Energy Variable.
	, NBT_ENERGY_EU					= "gt.energy.eu"				// Long containing a specified Energy Variable.
	, NBT_ENERGY_RU					= "gt.energy.ru"				// Long containing a specified Energy Variable.
	, NBT_ENERGY_KU					= "gt.energy.ku"				// Long containing a specified Energy Variable.
	, NBT_ENERGY_HU					= "gt.energy.hu"				// Long containing a specified Energy Variable.
	, NBT_ENERGY_LU					= "gt.energy.lu"				// Long containing a specified Energy Variable.
	, NBT_ENERGY_SU					= "gt.energy.su"				// Long containing a specified Energy Variable.
	, NBT_ENERGY_AU					= "gt.energy.au"				// Long containing a specified Energy Variable.
	, NBT_ENERGY_MJ					= "gt.energy.mj"				// Long containing a specified Energy Variable.
	, NBT_ENERGY_RF					= "gt.energy.rf"				// Long containing a specified Energy Variable.
	
	, NBT_CAPACITY					= "gt.capacity"					// Long containing a generic Energy Variable as Capacity.
	, NBT_CAPACITY_EU				= "gt.capacity.eu"				// Long containing a specified Energy Variable as Capacity.
	, NBT_CAPACITY_RU				= "gt.capacity.ru"				// Long containing a specified Energy Variable as Capacity.
	, NBT_CAPACITY_KU				= "gt.capacity.ku"				// Long containing a specified Energy Variable as Capacity.
	, NBT_CAPACITY_HU				= "gt.capacity.hu"				// Long containing a specified Energy Variable as Capacity.
	, NBT_CAPACITY_LU				= "gt.capacity.lu"				// Long containing a specified Energy Variable as Capacity.
	, NBT_CAPACITY_SU				= "gt.capacity.su"				// Long containing a specified Energy Variable as Capacity.
	, NBT_CAPACITY_AU				= "gt.capacity.au"				// Long containing a specified Energy Variable as Capacity.
	, NBT_CAPACITY_MJ				= "gt.capacity.mj"				// Long containing a specified Energy Variable as Capacity.
	, NBT_CAPACITY_RF				= "gt.capacity.rf"				// Long containing a specified Energy Variable as Capacity.
	;
	
	/** Contains used Potion Effects. IDs are to be set on API postInit. */
	public static class PotionsGT {
		public static int
		ID_RADIATION = -2,
		ID_HYPOTHERMIA = -3, ID_HEATSTROKE = -4, ID_FROSTBITE = -5, ID_DEHYDRATION = -6, ID_INSANITY = -7;
	}
	
	/** Contains GT Items and Item related References. */
	public static class ItemsGT {
		/** The Book MultiItem, in order to have a reference to it for creating Books. */
		public static Item sMultiItemBooks, sMultiItemCans;
		
		public static boolean addNEIRedirect(ItemStack aStack, ItemStack... aRedirects) {if (aStack == null) return F; ArrayListNoNulls<ItemStack> tList = sNEIRedirects.get(aStack); if (tList == null) sNEIRedirects.put(new ItemStackContainer(aStack), tList = new ArrayListNoNulls<ItemStack>()); return tList.addAll(Arrays.asList(aRedirects));}
		public static final Map<ItemStackContainer, ArrayListNoNulls<ItemStack>> sNEIRedirects = new ItemStackMap<ItemStackContainer, ArrayListNoNulls<ItemStack>>();
	}
	
	/** Contains GT Blocks and Block related References. */
	public static class BlocksGT {
		public static PrefixBlock
		casingMachine, casingMachineDouble, casingMachineQuadruple, casingMachineDense,
		blockGem, blockDust, blockIngot, blockSolid, blockPlate, blockPlateGem,
		crateGtGem, crateGtDust, crateGtIngot, crateGtPlate, crateGtPlateGem;
		
		/** Ore Blocks */
		public static PrefixBlock
		ore			, oreSandstone			, oreNetherrack			, oreEndstone		, oreRedgranite			, oreBlackgranite		, oreBasalt			, oreMarble			, oreUBIgneous[]		= new PrefixBlock[8], oreUBMetamorphic[]		= new PrefixBlock[8], oreUBSedimentary[]		= new PrefixBlock[8], oreAtumLimestone			, oreAtumSand, oreGravel, oreSand, oreRedSand, oreBedrock,
		oreBroken	, oreBrokenSandstone	, oreBrokenNetherrack	, oreBrokenEndstone	, oreBrokenRedgranite	, oreBrokenBlackgranite	, oreBrokenBasalt	, oreBrokenMarble	, oreBrokenUBIgneous[]	= new PrefixBlock[8], oreBrokenUBMetamorphic[]	= new PrefixBlock[8], oreBrokenUBSedimentary[]	= new PrefixBlock[8], oreBrokenAtumLimestone	,
		oreSmall	, oreSmallSandstone		, oreSmallNetherrack	, oreSmallEndstone	, oreSmallRedgranite	, oreSmallBlackgranite	, oreSmallBasalt	, oreSmallMarble	, oreSmallUBIgneous[]	= new PrefixBlock[8], oreSmallUBMetamorphic[]	= new PrefixBlock[8], oreSmallUBSedimentary[]	= new PrefixBlock[8], oreSmallAtumLimestone		, oreSmallAtumSand, oreSmallGravel, oreSmallSand, oreSmallRedSand, oreSmallBedrock;
		
		public static Block
		GraniteBlack, GraniteRed, Basalt, Marble, CFoam, CFoamFresh, Glass;
		
		// Yes, I know those assignments inside the Array are not actually working, but this gives a good overview of the Content, once it got initialised.
		/** Contains all the GT Type Stones. */
		public static Block[] stones = {GraniteBlack, GraniteRed, Basalt, Marble};
		
		/** Mappings for Stones to their correspondent Ores. No WildCard allowed! Register all 16 MetaData Values here instead. */
		public static Map<ItemStackContainer, IBlockPlacable>
		  stoneToNormalOres	= new ItemStackMap<ItemStackContainer, IBlockPlacable>()
		, stoneToBrokenOres	= new ItemStackMap<ItemStackContainer, IBlockPlacable>()
		, stoneToSmallOres	= new ItemStackMap<ItemStackContainer, IBlockPlacable>()
		;
	}
	
	/** Contains the Tool IDs for my MetaTool. */
	public static class ToolsGT {
		public static final short SWORD = 0, PICKAXE = 2, SHOVEL = 4, AXE = 6, HOE = 8, SAW = 10, HARDHAMMER = 12, SOFTHAMMER = 14, WRENCH = 16, FILE = 18, CROWBAR = 20, SCREWDRIVER = 22, MORTAR = 24, WIRECUTTER = 26, SCOOP = 28, BRANCHCUTTER = 30, UNIVERSALSPADE = 32, KNIFE = 34, BUTCHERYKNIFE = 36, SICKLE = 38, SENSE = 40, PLOW = 42, PLUNGER = 44, ROLLING_PIN = 46, CHISEL = 48, FLINT_AND_TINDER = 50, MONKEY_WRENCH = 52, BENDING_CYLINDER = 54, BENDING_CYLINDER_SMALL = 56, DOUBLE_AXE = 58, CONSTRUCTION_PICK = 60, MAGNIFYING_GLASS = 62, SCISSORS = 64, PINCERS = 66, DRILL_LV = 100, DRILL_MV = 102, DRILL_HV = 104, CHAINSAW_LV = 110, CHAINSAW_MV = 112, CHAINSAW_HV = 114, WRENCH_LV = 120, WRENCH_MV = 122, WRENCH_HV = 124, JACKHAMMER = 130, BUZZSAW = 140, SCREWDRIVER_LV = 150;
		public static MultiItemTool sMetaTool;
		
		private static final ItemStackSet<ItemStackContainer> TOOL_LIST = new ItemStackSet<ItemStackContainer>();
		private static final Map<String, ItemStackSet<ItemStackContainer>> TOOL_LISTS = new HashMap<String, ItemStackSet<ItemStackContainer>>();
		private static ItemStackSet<ItemStackContainer> get(String aToolType) {ItemStackSet<ItemStackContainer> rSet = TOOL_LISTS.get(aToolType); if (rSet == null) TOOL_LISTS.put(aToolType, rSet = new ItemStackSet<ItemStackContainer>()); return rSet;}
		public static boolean contains(String aToolType, ItemStack aStack) {return get(aToolType).contains(aStack, T);}
		public static boolean contains(String aToolType, ItemStackContainer aStack) {return get(aToolType).contains(aStack, T);}
		public static boolean add(String aToolType, ItemStackContainer aStack) {if (TOOL_LIST.add(aStack)) return get(aToolType).add(aStack); return F;}
		public static boolean add(ItemStackContainer aStack, String aToolType) {if (TOOL_LIST.add(aStack)) return get(aToolType).add(aStack); return F;}
		public static boolean add(String aToolType, ItemStack aStack) {if (TOOL_LIST.add(aStack)) return get(aToolType).add(aStack); return F;}
		public static boolean add(ItemStack aStack, String aToolType) {if (TOOL_LIST.add(aStack)) return get(aToolType).add(aStack); return F;}
	}
	
	/** Contains TextureSet Indices for non OreDictPrefixed Stuff. */
	public static class IconsGT {
		public static int INDEX_BLOCK_MOLTEN = 0, INDEX_BLOCK_PIPE_SIDE = 0;
	}
	
	/** Files */
	public static class DirectoriesGT {
		public static File
		MINECRAFT,
		LOGS,
		CONFIG,
		CONFIG_GT,
		CONFIG_RECIPES;
	}
	
	/** Configs */
	public static class ConfigsGT {
		public static Config
		CLIENT = null,
		SPECIAL = null,
		RECIPES = null,
		MACHINES = null,
		WORLDGEN = null,
		MATERIAL = null,
		OVERPOWERED = null,
		OREPROCESSING = null;
	}
	
	/** Class Containing MOD ID Strings used in GT, since they are very common Parameters. */
	public static class ModNames {
		/** MOD ID Strings */
		@SuppressWarnings("hiding")
		public static final String
		  MC				= "Minecraft"
		  
	    , GT				= "GregTech"
	    , GAPI				= "Greg-API"
	    , GAPI_POST			= "Greg-API-Post"
	    , API				= "Greg-API"
	    
	    , IC2				= "IndustrialCraft 2"
	    , NC				= "Nuclear Control"
	    
	    , FMB				= "Forge Microblocks"
	    
	    , TC				= "Thaumcraft"
	    
	    , TF				= "Twilight Forest"
	    
	    , RP				= "Redpower"
	    
	    , RC				= "Railcraft"
	    
	    , IE				= "Immersive Engineering"
	    
	    , TE				= "Thermal Expansion"
	    
	    , AE 				= "Applied Energistics"
	    
	    , TFC				= "TerraFirmaCraft"
	    
	    , MFR				= "MineFactoryReloaded"
	    
	    , MET				= "Metallurgy"
	    
	    , ATUM				= "Atum"
	    
	    , CHSL				= "Chisel"
	    
	    , UB				= "Underground Biomes"
	    , PFAA				= "Per Fabrica Ad Astra"
	    
	    , FR				= "Forestry"
	    
	    , MgC				= "Magneticraft"
	    
	    , DRGN				= "Dragon API"
	    , RoC				= "RotaryCraft"
	    , ReC				= "ReactorCraft"
	    , ElC				= "ElectriCraft"
	    
	    , OC				= "Open Computers"
	    , CC				= "ComputerCraft"
	    
	    , HaC				= "HarvestCraft"
	    , APC				= "AppleCore"
	    , ENVM				= "Enviromine"
	    , MaCr				= "Magical Crops"
	    
	    , GaEn				= "Ganys End"
	    , GaSu				= "Ganys Surface"
	    , GaNe				= "Ganys Nether"
	    
	    , BC				= "BuildCraft"
	    , BC_CORE			= "BuildCraft Core"
	    , BC_SILICON		= "BuildCraft Silicon"
	    , BC_TRANSPORT		= "BuildCraft Transport"
	    , BC_FACTORY		= "BuildCraft Factory"
	    , BC_ENERGY			= "BuildCraft Energy"
	    , BC_BUILDERS		= "BuildCraft Builders"
	    
	    , COFH_API			= "CoFHAPI"
	    , COFH_API_ENERGY	= "CoFHAPI Energy"
	    
	    , OB				= "OpenBlocks"
	    
	    , GC				= "Galacticraft"
	    , GC_CORE			= "Galacticraft Core"
	    , GC_MARS			= "Galacticraft Mars"
	    , GC_PLANETS		= "Galacticraft Planets"
	    ;
	}
	
	/** Class Containing MOD ID Strings used in GT, since they are very common Parameters. */
	public static class ModIDs {
		/** MOD ID Strings */
		@SuppressWarnings("hiding")
		public static final String
	      GT				= "gregtech"
	    , GAPI				= "gregapi"
	    , GAPI_POST			= "gregapi_post"
	    , API				= "gregapi"
	    
	    , IC2				= "IC2"
	    , NC				= "IC2NuclearControl"
	    
	    , FMB				= "ForgeMicroblock"
	    
	    , TC				= "Thaumcraft"
	    
	    , TF				= "TwilightForest"
	    
	    , RC				= "Railcraft"
	    
	    , IE				= "ImmersiveEngineering"
	    
	    , TE				= "ThermalExpansion"
	    , TE_CORE			= "ThermalFoundation"
	    
	    , AE 				= "appliedenergistics2"
	    
	    , TFC				= "terrafirmacraft"
	    
	    , ATUM				= "atum"
	    
	    , CHSL				= "chisel"
	    
	    , UB				= "UndergroundBiomes"
	    , PFAA				= "PFAAGeologica"
	    
	    , FR				= "Forestry"
	    
	    , MgC				= "Magneticraft"
	    
	    , DRGN				= "DragonAPI"
	    , RoC				= "RotaryCraft"
	    , ReC				= "ReactorCraft"
	    , ElC				= "ElectriCraft"
	    
	    , OC				= "OpenComputers"
	    , CC				= "ComputerCraft"
	    
	    , HaC				= "harvestcraft"
	    , APC				= "AppleCore"
	    , ENVM				= "enviromine"
	    , MaCr				= "magicalcrops"
	    
	    , GaEn				= "ganysend"
	    , GaSu				= "ganyssurface"
	    , GaNe				= "ganysnether"
	    
	    , BC				= "BuildCraft|Core"
	    , BC_CORE			= "BuildCraft|Core"
	    , BC_SILICON		= "BuildCraft|Silicon"
	    , BC_TRANSPORT		= "BuildCraft|Transport"
	    , BC_FACTORY		= "BuildCraft|Factory"
	    , BC_ENERGY			= "BuildCraft|Energy"
	    , BC_BUILDERS		= "BuildCraft|Builders"
	    
	    , COFH_API			= "CoFHAPI"
	    , COFH_API_ENERGY	= "CoFHAPI|energy"
	    
	    , OB				= "OpenBlocks"
	    
	    , GC				= "GalacticraftCore"
	    , GC_CORE			= "GalacticraftCore"
	    , GC_MARS			= "GalacticraftMars"
	    , GC_PLANETS		= "GalacticraftPlanets"
	    ;
	}
	
	/** Class Containing Sound Strings used in GT. */
	public static class SFX {
		/** Sound Strings */
		public static final String
	      MC_BREAK				= "random.break"
	    , MC_ANVIL_USE			= "random.anvil_use"
	    , MC_ANVIL_BREAK		= "random.anvil_break"
	    , MC_CLICK				= "random.click"
	    , MC_FIZZ				= "random.fizz"
	    , MC_EXPLODE			= "random.explode"
	    , MC_IGNITE				= "fire.ignite"
	    , MC_DIG_ROCK			= "dig.stone"
	    , MC_DIG_SAND			= "dig.sand"
	    , MC_LIQUID_WATER		= "liquid.water"
	    , MC_HMM				= "mob.villager.idle"
	    , MC_AHA				= "mob.villager.haggle"
	    , MC_SHEARS				= "mob.sheep.shear"
	    
	    , IC_WRENCH				= ModIDs.IC2.toLowerCase() + ":" + "tools.Wrench"
	    , IC_TRAMPOLINE			= ModIDs.IC2.toLowerCase() + ":" + "tools.RubberTrampoline"
	    , IC_PAINT				= ModIDs.IC2.toLowerCase() + ":" + "tools.Painter"
	    , IC_SPRAY				= ModIDs.IC2.toLowerCase() + ":" + "tools.Painter"
	    , IC_BATTERYUSE			= ModIDs.IC2.toLowerCase() + ":" + "tools.BatteryUse"
	    , IC_SOLDERING			= ModIDs.IC2.toLowerCase() + ":" + "tools.BatteryUse"
	    , IC_CHAINSAW_01		= ModIDs.IC2.toLowerCase() + ":" + "tools.chainsaw.ChainsawUseOne"
	    , IC_CHAINSAW_02		= ModIDs.IC2.toLowerCase() + ":" + "tools.chainsaw.ChainsawUseTwo"
	    , IC_DRILL_SOFT			= ModIDs.IC2.toLowerCase() + ":" + "tools.drill.DrillSoft"
	    , IC_DRILL_HARD			= ModIDs.IC2.toLowerCase() + ":" + "tools.drill.DrillHard"
	    , IC_SCANNER			= ModIDs.IC2.toLowerCase() + ":" + "tools.ODScanner"
	    
		, IC_MACHINE_EXTRACTOR	= ModIDs.IC2.toLowerCase() + ":" + "machines.ExtractorOp"
		, IC_MACHINE_MACERATOR	= ModIDs.IC2.toLowerCase() + ":" + "machines.MaceratorOp"
		, IC_MACHINE_INDUCTION	= ModIDs.IC2.toLowerCase() + ":" + "machines.InductionLoop"
		, IC_MACHINE_COMPRESSOR	= ModIDs.IC2.toLowerCase() + ":" + "machines.CompressorOp"
		, IC_MACHINE_RECYCLER	= ModIDs.IC2.toLowerCase() + ":" + "machines.RecyclerOp"
		, IC_MACHINE_MINER		= ModIDs.IC2.toLowerCase() + ":" + "machines.MinerOp"
		, IC_MACHINE_PUMP		= ModIDs.IC2.toLowerCase() + ":" + "machines.PumpOp"
		, IC_MACHINE_FURNACE	= ModIDs.IC2.toLowerCase() + ":" + "machines.ElectroFurnaceLoop"
		, IC_MACHINE_OVERLOAD	= ModIDs.IC2.toLowerCase() + ":" + "machines.MachineOverload"
		, IC_MACHINE_INTERRUPT	= ModIDs.IC2.toLowerCase() + ":" + "machines.InterruptOne"
		, IC_MACHINE_KA_CHING	= ModIDs.IC2.toLowerCase() + ":" + "machines.KaChing"
		, IC_MACHINE_MAGNETIZER	= ModIDs.IC2.toLowerCase() + ":" + "machines.MagnetizerLoop"
		
	   	, MISSING			= null;
	}
	
	/** File Paths and Resource Paths */
    public static final String
      TEX_DIR					= "textures/"
    , TEX_DIR_GUI				= TEX_DIR + "gui/"
    , TEX_DIR_ITEM				= TEX_DIR + "items/"
    , TEX_DIR_MODEL				= TEX_DIR + "model/"
    , TEX_DIR_BLOCK				= TEX_DIR + "blocks/"
    , TEX_DIR_ENTITY			= TEX_DIR + "entity/"
    , TEX_DIR_ASPECTS			= TEX_DIR + "aspects/"
    
    , RES_PATH					= ModIDs.GT  + ":" + TEX_DIR
    , RES_PATH_GUI				= ModIDs.GT  + ":" + TEX_DIR_GUI
    , RES_PATH_ITEM				= ModIDs.GT  + ":"
    , RES_PATH_BLOCK			= ModIDs.GT  + ":"
    , RES_PATH_ENTITY			= ModIDs.GT  + ":" + TEX_DIR_ENTITY
    , RES_PATH_API_ITEM			= ModIDs.GAPI + ":"
    , RES_PATH_API_BLOCK		= ModIDs.GAPI + ":"
    , RES_PATH_ASPECTS			= ModIDs.GAPI + ":" + TEX_DIR_ASPECTS
    
    , RES_PATH_IC2				= ModIDs.IC2.toLowerCase() + ":"
    ;
}